'use strict';

var path = require('path');
require('assert');
var helperModuleImports = require('@babel/helper-module-imports');
require('babel-template');

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

var NamingRule = {
  LITTLE_CAMEL: 'little-camel',
  BIG_CAMEL: 'big-camel',
  UNDERLINE: 'underline',
  DASH: 'dash',
  FUNCTION: 'FUNCTION'
};

/**
 * @desc 封装了一些项目常用方法.
 */
// 内部函数, 用于判断对象类型
function _getClass(object) {
  return Object.prototype.toString.call(object).match(/^\[object\s(.*)\]$/)[1];
}

function isArray(obj) {
  return _getClass(obj).toLowerCase() === 'array';
}
function isString(obj) {
  return _getClass(obj).toLowerCase() === 'string';
}
function isObject(obj) {
  return _getClass(obj).toLowerCase() === 'object';
}
function isNumber(obj) {
  return _getClass(obj).toLowerCase() === 'number';
}
function isFunction(obj) {
  return _getClass(obj).toLowerCase() === 'function';
}
/**
 * @desc 判断参数是否为空, 包括null, undefined, [], '', {}
 * @param {object} obj 需判断的对象
 */

function isEmpty(obj) {
  var empty = false;

  if (obj === null || obj === undefined) {
    // null and undefined
    empty = true;
  } else if ((isArray(obj) || isString(obj)) && obj.length === 0) {
    empty = true;
  } else if (isObject(obj)) {
    var hasProp = false;

    for (var prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        hasProp = true;
        break;
      }
    }

    if (!hasProp) {
      empty = true;
    }
  } else if (isNumber(obj) && isNaN(obj)) {
    empty = true;
  }

  return empty;
}
/**
 * @desc 判断参数是否为空字符串, 比isEmpty()多判断字符串中全是空格的情况, 如: '   '.
 * @param {string} str 需判断的字符串
 */

function isBlank(str) {
  if (isEmpty(str)) {
    return true;
  } else if (isString(str) && str.trim().length === 0) {
    return true;
  }

  return false;
}
/**
 * @desc 判断参数是否不为空字符串
 */

function isNotBlank(obj) {
  return !isBlank(obj);
}

var ImportSpecifierVisitors = {
  ImportSpecifier: function ImportSpecifier(path$$1, state) {
    var specifiers = state.specifiers;
    var _path$node = path$$1.node,
        imported = _path$node.imported,
        local = _path$node.local;
    specifiers.push({
      imported: imported.name,
      local: local.name
    });
  },
  ImportDefaultSpecifier: function ImportDefaultSpecifier(path$$1, state) {// TODO: "foo" in import foo from 'mod'.
  },
  ImportNamespaceSpecifier: function ImportNamespaceSpecifier(path$$1, state) {// TODO: "* as foo" in import * as foo from 'mod'.
  }
};

function isTargetImport(source, target) {
  return source === target;
}

function toLittleCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[A-Z]/, function (match) {
    return match.toLowerCase();
  });
}

function toBigCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[a-z]/, function (match) {
    return match.toUpperCase();
  });
}

function toDash() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '-' + _str : _str;
  });
}

function toUnderline() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '_' + _str : _str;
  });
}

function buildNameByRule(name, rule) {
  switch (rule) {
    case NamingRule.LITTLE_CAMEL:
      // 小驼峰
      name = toLittleCamel(name);
      break;

    case NamingRule.BIG_CAMEL:
      // 大驼峰
      name = toBigCamel(name);
      break;

    case NamingRule.DASH:
      // 中横线
      name = toDash(name);
      break;

    case NamingRule.UNDERLINE:
      // 下划线
      name = toUnderline(name);
      break;

    default:
      if (isString(rule)) {
        // other string
        name = rule.trim();
      } else if (isFunction(rule)) {
        name = rule(name);
      }

  }

  if (!name.startsWith('/')) {
    name = '/' + name;
  }

  return name;
}

function buildPath() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var paths = args.map(function (path$$1) {
    return path$$1 || '';
  });
  return path.join.apply(void 0, _toConsumableArray(paths)).replace(/\\/g, '/');
} // Entry


function visitor (_ref) {
  var t = _ref.types;
  return {
    pre: function pre(state) {},
    visitor: {
      ImportDeclaration: function ImportDeclaration(path$$1, state) {
        var source = path$$1.node.source.value;

        var _Object$assign = Object.assign({
          directory: 'lib',
          namingRule: NamingRule.BIG_CAMEL,
          importDefault: true
        }, state.opts),
            library = _Object$assign.library,
            directory = _Object$assign.directory,
            namingRule = _Object$assign.namingRule,
            importDefault = _Object$assign.importDefault,
            style = _Object$assign.style;

        if (!library) {
          throw new Error('library should be provided');
        }

        if (isTargetImport(source, library)) {
          var _state = {
            specifiers: []
          };
          path$$1.traverse(ImportSpecifierVisitors, _state);

          if (_state.specifiers) {
            // const componentTemplate = template(`
            //     var LOCAL_NAME = require(SOURCE);
            // `);
            // var newPath = [];
            _state.specifiers.forEach(function (specifier) {
              // handle component node
              // var componentDirectory = buildDirectory(library, directory);
              var componentName = buildNameByRule(specifier.imported, namingRule);
              var componentPath = buildPath(library, directory, componentName); // var componentNode = componentTemplate({
              //     LOCAL_NAME: t.identifier(specifier.local),
              //     SOURCE: t.stringLiteral(componentSource)
              // });

              if (importDefault) {
                helperModuleImports.addDefault(path$$1, componentPath);
              } else {
                helperModuleImports.addNamed(path$$1, specifier.local, componentPath);
              } // newPath.push(componentNode);
              // handle style node


              if (style) {
                // const styleTemplate = template(`
                //     import SOURCE;
                // `);
                var _Object$assign2 = Object.assign({}, style),
                    _directory = _Object$assign2.directory,
                    _namingRule = _Object$assign2.namingRule,
                    ext = _Object$assign2.ext;

                var styleDirectory = buildPath(library, directory);
                var styleName = '';
                var extension = '';

                if (_directory) {
                  if (_directory.startsWith('/')) {
                    // 从根路径开始设置样式路径, 如: '/less/component'.
                    // styleDirectory = buildDirectory(library, _directory);
                    styleDirectory = buildPath(library, _directory);
                  } else {
                    // 在 componentSource 后追加路径, 如: 'style'.
                    styleDirectory = buildPath(componentPath, _directory);
                  }
                }

                if (_namingRule) {
                  styleName = buildNameByRule(specifier.imported, _namingRule);
                }

                if (isNotBlank(ext)) {
                  extension = ext.trim().startsWith('.') ? ext : '.' + ext;
                } // /antd/lib/button/style(/index.js)        directory = 'style'
                // /antd/lib/button/style/index.less        directory = 'style', namingRule = 'index', extension = 'less'
                // /elemental-ui/less/component/Alert.less  directory = '/less/component' namingRule = 'big-camel', extension = 'less'


                var stylePath = buildPath(styleDirectory, styleName + extension); // let styleNode = styleTemplate({
                //     SOURCE: t.stringLiteral(styleSource)
                // });

                var styleNode = helperModuleImports.addSideEffect(path$$1, stylePath); // newPath.push(styleNode);
              }
            });

            path$$1.remove(); // path.replaceWithMultiple(newPath);
          }
        }
      }
    },
    post: function post(state) {}
  };
}

module.exports = visitor;
